/* 
Base code for joint class with Applied Parallel computing
Using a framebuffer as a basis to implement Gaussian blur in GLSL
Currently will make 2 FBOs and textures (only uses one in base code)
and writes out frame as a .png (Texture_output.png)
upper right quad on screen is blue-er to show fragment shader effect on texture

Winter 2017 - ZJW (Piddington texture write)
Look for "TODO" in this file and write new shaders
*/

#include <iostream>
#define GLEW_STATIC
#include <GL/glew.h>
#include <math.h>
#include <time.h>
#include <GLFW/glfw3.h>

#include "GLSL.h"
#include "Program.h"
#include "MatrixStack.h"
#include "Shape.h"
#include "Texture.h"
#include "GLTextureWriter.h"

// value_ptr for glm
#include <glm/gtc/type_ptr.hpp>
#include <glm/gtc/matrix_transform.hpp>

using namespace std;
using namespace glm;

GLFWwindow *window; // Main application window
string RESOURCE_DIR = ""; // Where the resources are loaded from
shared_ptr<Program> prog, seaweedProg, aquariumProg, coralProg, pebbleProg;
shared_ptr<Texture> waterTex, seaweedTex, coralTex, pebbleTex;
shared_ptr<Shape> castle, cube, plane, sphere, cone, cylinder;

int g_width = 1200;
int g_height = 1200;
float speed = 4;
float xLightPos = 25.6f;
float yLightPos = 40.0f;
float zLightPos = 40.0f;

vec3 eye = vec3(0, 45, 40);
vec3 lookAtVec = vec3(0, -0.8, -1);
vec3 upVector = vec3(0, 1, 0);
vec3 view;
vec3 vecW;
vec3 vecU;
float phi = -(M_PI/4.0f);
float theta = (M_PI*3)/2.0f;
double xOldMouse;
double yOldMouse;

int seaweedNum = 30;
vec3 seaweedPositions[30];
float seaweedRotations[30];

int coralNum = 10;
vec3 coralPositions[10];
float coralRotations[10];
int coralTexIndex = 0;
shared_ptr<Texture> coralTextures[2];

int pebbleIdx = 1;

int dark = 0;
bool outsideTank = true;

//forward declaring a useful function listed later
void SetMaterial(int i);

// sets random locations for coral and seaweed
void setRandomLocations() {
	int xPos = 0;
	int zPos = 0;
	srand(time(NULL));
	for(int i = 0; i < seaweedNum; i++) {
		while (xPos >= -16 && xPos <= 16 && zPos >= -20 && zPos <= 0) {
			xPos = rand() % 45 + (-20);
			zPos = rand() % 33 + (-25);
		}
		seaweedPositions[i] = vec3(xPos, 10, zPos);
		seaweedRotations[i] = radians((float)rand());
		xPos = 0;
		zPos = 0;
	}
	for(int i = 0; i < coralNum; i++) {
		while (xPos >= -16 && xPos <= 16 && zPos >= -20 && zPos <= 0) {
			xPos = rand() % 40 + (-20);
			zPos = rand() % 26 + (-22);
		}
		coralPositions[i] = vec3(xPos, 8, zPos);
		coralRotations[i] = radians((float)rand());
		xPos = 0;
		zPos = 0;
	}
}

/* lots of initialization to set up the opengl state and data */
static void initGL()
{
	GLSL::checkVersion();
  	int width, height;
  	glfwGetFramebufferSize(window, &width, &height);
  	glfwGetCursorPos(window, &xOldMouse, &yOldMouse);

	// Set background color.
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);

	// Enable z-buffer test.
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_CULL_FACE);

	/////////////////////////////////////////////////////////////////////////////////
    // obj file setup
    /////////////////////////////////////////////////////////////////////////////////
	
	castle = make_shared<Shape>();
	castle->loadMesh(RESOURCE_DIR + "wooden_castle.obj");
	castle->resize();
	castle->init();

	cube = make_shared<Shape>();
	cube->loadMesh(RESOURCE_DIR + "cube.obj");
	cube->resize();
	cube->init();

	sphere = make_shared<Shape>();
	sphere->loadMesh(RESOURCE_DIR + "sphere.obj");
	sphere->resize();
	sphere->init();

	cone = make_shared<Shape>();
	cone->loadMesh(RESOURCE_DIR + "cone.obj");
	cone->resize();
	cone->init();

	cylinder = make_shared<Shape>();
	cylinder->loadMesh(RESOURCE_DIR + "cylinder.obj");
	cylinder->resize();
	cylinder->init();

	plane = make_shared<Shape>();
	plane->loadMesh(RESOURCE_DIR + "highPolyPlane.obj");
	plane->resize();
	plane->init();

	/////////////////////////////////////////////////////////////////////////////////
    // program setup
    /////////////////////////////////////////////////////////////////////////////////

	// Initialize the GLSL program to render the obj
	prog = make_shared<Program>();
	prog->setVerbose(true);
	prog->setShaderNames(RESOURCE_DIR + "simple_vert.glsl", RESOURCE_DIR + "simple_frag.glsl");
	prog->init();
	prog->addUniform("P");
	prog->addUniform("MV");
	prog->addUniform("V");
	prog->addUniform("MatAmb");
	prog->addUniform("MatDif");
	prog->addUniform("MatSpec");
	prog->addUniform("light");
	prog->addUniform("shine");
	prog->addAttribute("vertPos");
	prog->addAttribute("vertNor");
	prog->addAttribute("vertTex");

	// Initialize seaweed program
	seaweedProg = make_shared<Program>();
	seaweedProg->setVerbose(true);
	seaweedProg->setShaderNames(RESOURCE_DIR + "tex_vert.glsl", RESOURCE_DIR + "seaweed_frag.glsl");
	seaweedProg->init();
	seaweedProg->addUniform("P");
	seaweedProg->addUniform("MV");
	seaweedProg->addUniform("V");
	seaweedProg->addUniform("seaweedTex");
	seaweedProg->addUniform("sysTime");
	seaweedProg->addAttribute("vertPos");
	seaweedProg->addAttribute("vertNor");
	seaweedProg->addAttribute("vertTex");

	// Initialize coral program
	coralProg = make_shared<Program>();
	coralProg->setVerbose(true);
	coralProg->setShaderNames(RESOURCE_DIR + "tex_vert.glsl", RESOURCE_DIR + "coral_frag.glsl");
	coralProg->init();
	coralProg->addUniform("P");
	coralProg->addUniform("MV");
	coralProg->addUniform("V");
	coralProg->addUniform("coralTex");
	coralProg->addUniform("sysTime");
	coralProg->addAttribute("vertPos");
	coralProg->addAttribute("vertNor");
	coralProg->addAttribute("vertTex");

	// Initialize pebble program
	pebbleProg = make_shared<Program>();
	pebbleProg->setVerbose(true);
	pebbleProg->setShaderNames(RESOURCE_DIR + "tex_vert.glsl", RESOURCE_DIR + "pebble_frag.glsl");
	pebbleProg->init();
	pebbleProg->addUniform("P");
	pebbleProg->addUniform("V");
	pebbleProg->addUniform("MV");
	pebbleProg->addUniform("texBuf");
	pebbleProg->addUniform("resolution");
	pebbleProg->addUniform("sysTime");
	pebbleProg->addUniform("dark");
	pebbleProg->addAttribute("vertPos");
	pebbleProg->addAttribute("vertNor");
	pebbleProg->addAttribute("vertTex");

	// Initialize tank wall program
	aquariumProg = make_shared<Program>();
	aquariumProg->setVerbose(true);
	aquariumProg->setShaderNames(RESOURCE_DIR + "tex_vert.glsl", RESOURCE_DIR + "aquarium_frag.glsl");
	aquariumProg->init();
	aquariumProg->addUniform("P");
	aquariumProg->addUniform("MV");
	aquariumProg->addUniform("V");
	aquariumProg->addUniform("waterTex");
	aquariumProg->addUniform("sysTime");
	aquariumProg->addUniform("resolution");
	aquariumProg->addUniform("dark");
	aquariumProg->addUniform("topRipple");
	aquariumProg->addAttribute("vertPos");
	aquariumProg->addAttribute("vertNor");
	aquariumProg->addAttribute("vertTex");

	// set up random location arrays for seawead and coral
	setRandomLocations();

	/////////////////////////////////////////////////////////////////////////////////
    // texture setup
    /////////////////////////////////////////////////////////////////////////////////

	// texture for water
	waterTex = make_shared<Texture>();
  	waterTex->setFilename(RESOURCE_DIR + "ocean.jpg");
    waterTex->init();
    waterTex->setUnit(0);
    waterTex->setWrapModes(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);

    // texture for tank floor
    pebbleTex = make_shared<Texture>();
  	pebbleTex->setFilename(RESOURCE_DIR + "pebbles1.jpg");
    pebbleTex->init();
    pebbleTex->setUnit(1);
    pebbleTex->setWrapModes(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);

    // texture for seaweed
    seaweedTex = make_shared<Texture>();
  	seaweedTex->setFilename(RESOURCE_DIR + "seaweed.jpg");
    seaweedTex->init();
    seaweedTex->setUnit(2);
    seaweedTex->setWrapModes(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);

    // texture for coral
    coralTex = make_shared<Texture>();
  	coralTex->setFilename(RESOURCE_DIR + "redCoral.jpg");
    coralTex->init();
    coralTex->setUnit(2);
    coralTex->setWrapModes(GL_REPEAT, GL_REPEAT);

}

/* The render loop - this function is called repeatedly during the OGL program run */
static void render()
{
	// Get current frame buffer size.
	int width, height;
	glfwGetFramebufferSize(window, &width, &height);
	glViewport(0, 0, width, height);
	
	//set up to render to buffer
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
	// Clear framebuffer.
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/* Leave this code to just draw the meshes alone */
	//Use the matrix stack for Lab 6
   float aspect = width/(float)height;

   // Create the matrix stacks - please leave these alone for now
   auto P = make_shared<MatrixStack>();
   auto MV = make_shared<MatrixStack>();
   auto V = make_shared<MatrixStack>();

   // setup camera
   view = lookAtVec;
   vecW = normalize(view);
   vecU = cross(upVector, -1.0f*vecW);

   V->pushMatrix();
   V->lookAt(eye, eye + lookAtVec, upVector);

   // Apply perspective projection.
   P->pushMatrix();
   P->perspective(45.0f, aspect, 0.01f, 100.0f);

   /////////////////////////////////////////////////////////////////////////////////
   // castle and fish program
   /////////////////////////////////////////////////////////////////////////////////

   prog->bind();

   glUniformMatrix4fv(prog->getUniform("V"), 1, GL_FALSE, value_ptr(V->topMatrix()));
   glUniformMatrix4fv(prog->getUniform("P"), 1, GL_FALSE, value_ptr(P->topMatrix()));
   glUniform3f(prog->getUniform("light"), xLightPos, yLightPos, zLightPos);
   float sysTime = glfwGetTime();

   //castle
   MV->pushMatrix();
     MV->loadIdentity();
	  MV->translate(vec3(0, 7, -10));
	  /* draw left mesh */	
		MV->pushMatrix();
		  SetMaterial(7);
		  MV->scale(vec3(8, 12, 8));
	  	  glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
	  	  castle->draw(prog);
   	MV->popMatrix();
   MV->popMatrix();

   // fish
   for (int i = 1; i < 6; i++) {
	   	MV->pushMatrix();
	   	  MV->loadIdentity();
	   	  MV->translate(vec3(0, 0, -10.5));
	   	  MV->rotate(i*2 + sysTime+(i/2.0f), vec3(0, 1, 0));
	   	  MV->translate(vec3(8+(i*1.5), 2+i*2, -8));
	   	  MV->rotate(-M_PI/4.0f, vec3(0, 1, 0));
	   	  MV->pushMatrix();
	   	  	  MV->translate(vec3(2, 0, 0));
	   	  	  MV->rotate(M_PI/2, vec3(0, 0, 1));
	   	  	  MV->rotate(sin(sysTime*7), vec3(1, 0, 0));
	   	  	  MV->scale(vec3(1*i, 1, 0));
	   	      SetMaterial(i%6+1);
	   	      glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	  cone->draw(prog);
		  MV->popMatrix();
		  MV->pushMatrix();
		  	MV->translate(vec3(-1, 0, 0.8));
		  	MV->scale(vec3(0.3, 0.3, 0.3));
		  	SetMaterial(10);
		  	glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	sphere->draw(prog);
		  MV->popMatrix();
		  MV->pushMatrix();
		  	MV->translate(vec3(-1, 0, -0.8));
		  	MV->scale(vec3(0.3, 0.3, 0.3));
		  	SetMaterial(10);
		  	glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	sphere->draw(prog);
		  MV->popMatrix();
		   MV->pushMatrix();
		  	MV->translate(vec3(-1, 0, 1));
		  	MV->scale(vec3(0.2, 0.2, 0.2));
		  	SetMaterial(9);
		  	glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	sphere->draw(prog);
		  MV->popMatrix();
		  MV->pushMatrix();
		  	MV->translate(vec3(-1, 0, -1));
		  	MV->scale(vec3(0.2, 0.2, 0.2));
		  	SetMaterial(9);
		  	glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	sphere->draw(prog);
		  MV->popMatrix();
		  MV->scale(vec3(2.5, 1*(i/2.0f), 1));
		  SetMaterial(i%6+1);
	   	  glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  sphere->draw(prog);
		MV->popMatrix();
	}

	// fish
	for (int i = 1; i < 6; i++) {
	   	MV->pushMatrix();
	   	  MV->loadIdentity();
	   	  MV->translate(vec3(0, 0, -10.5));
	   	  MV->rotate(i*2 + -1*sysTime+(i/2.0f), vec3(0, 1, 0));
	   	  MV->translate(vec3(8+(i*1.5), 16-i*2, -8));
	   	  MV->rotate(-3*M_PI/2.0f, vec3(0, 1, 0));
	   	  MV->pushMatrix();
	   	  	  MV->translate(vec3(2, 0, 0));
	   	  	  MV->rotate(M_PI/2, vec3(0, 0, 1));
	   	  	  MV->rotate(sin(sysTime*7), vec3(1, 0, 0));
	   	  	  MV->scale(vec3(1*i, 1, 0));
	   	      SetMaterial((i+5)%6+4);
	   	      glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	  cone->draw(prog);
		  MV->popMatrix();
		  MV->pushMatrix();
		  	MV->translate(vec3(-1, 0, 0.8));
		  	MV->scale(vec3(0.3, 0.3, 0.3));
		  	SetMaterial(10);
		  	glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	sphere->draw(prog);
		  MV->popMatrix();
		  MV->pushMatrix();
		  	MV->translate(vec3(-1, 0, -0.8));
		  	MV->scale(vec3(0.3, 0.3, 0.3));
		  	SetMaterial(10);
		  	glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	sphere->draw(prog);
		  MV->popMatrix();
		   MV->pushMatrix();
		  	MV->translate(vec3(-1, 0, 1));
		  	MV->scale(vec3(0.2, 0.2, 0.2));
		  	SetMaterial(9);
		  	glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	sphere->draw(prog);
		  MV->popMatrix();
		  MV->pushMatrix();
		  	MV->translate(vec3(-1, 0, -1));
		  	MV->scale(vec3(0.2, 0.2, 0.2));
		  	SetMaterial(9);
		  	glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	sphere->draw(prog);
		  MV->popMatrix();
		  MV->scale(vec3(2.5, 1*(i/2.0f), 1));
		  SetMaterial((i+5)%6+4);
	   	  glUniformMatrix4fv(prog->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  sphere->draw(prog);
		MV->popMatrix();
	}

   prog->unbind();

   /////////////////////////////////////////////////////////////////////////////////
   // tank floor program
   /////////////////////////////////////////////////////////////////////////////////

   pebbleProg->bind();
   pebbleTex->bind(pebbleProg->getUniform("texBuf"));

   glUniform1f(pebbleProg->getUniform("sysTime"), sysTime);
   glUniform2f(pebbleProg->getUniform("resolution"), width, height);
   glUniform1i(pebbleProg->getUniform("dark"), dark);

   glUniformMatrix4fv(pebbleProg->getUniform("P"), 1, GL_FALSE, value_ptr(P->topMatrix()));
   glUniformMatrix4fv(pebbleProg->getUniform("V"), 1, GL_FALSE, value_ptr(V->topMatrix()));
	   //bottom wall
	   MV->pushMatrix();
	   	MV->loadIdentity();
	   	MV->translate(vec3(0, 0.1, -10));
	   	MV->pushMatrix();
	   		MV->scale(vec3(30, 0, 20));
	   		glUniformMatrix4fv(pebbleProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()));
		  	plane->draw(pebbleProg);
	   	  MV->popMatrix();
	   MV->popMatrix();

	pebbleProg->unbind();

   /////////////////////////////////////////////////////////////////////////////////
   // coral program
   /////////////////////////////////////////////////////////////////////////////////

   coralProg->bind();
   coralTex->bind(coralProg->getUniform("coralTex"));

   glUniformMatrix4fv(coralProg->getUniform("P"), 1, GL_FALSE, value_ptr(P->topMatrix()));
   glUniformMatrix4fv(coralProg->getUniform("V"), 1, GL_FALSE, value_ptr(V->topMatrix()));
   glUniform1f(coralProg->getUniform("sysTime"), sysTime);

	for (int i = 0; i < coralNum; i++) {   

		   MV->pushMatrix();
			  MV->loadIdentity();
			  MV->translate(coralPositions[i]);
			  MV->rotate(-M_PI/2.0f, vec3(0, 0, 1));
			  MV->rotate(coralRotations[i], vec3(1, 0, 0));
			  MV->pushMatrix();
				MV->scale(vec3(8, 0, 8));
			    glUniformMatrix4fv(coralProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()));
			  	plane->draw(coralProg);
		   	MV->popMatrix();
		   MV->popMatrix();

		   MV->pushMatrix();
			  MV->loadIdentity();
			  MV->translate(coralPositions[i]);
			  MV->rotate(-M_PI/2.0f, vec3(0, 0, 1));
			  MV->rotate(coralRotations[i]/4.0f, vec3(1, 0, 0));
			  MV->pushMatrix();
				MV->scale(vec3(8, 0, 8));
			    glUniformMatrix4fv(coralProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()));
			  	plane->draw(coralProg);
		   	MV->popMatrix();
		   MV->popMatrix();
    }
   coralProg->unbind();

   /////////////////////////////////////////////////////////////////////////////////
   // seaweed program
   /////////////////////////////////////////////////////////////////////////////////

   seaweedProg->bind();
   seaweedTex->bind(seaweedProg->getUniform("seaweedTex"));

   glUniformMatrix4fv(seaweedProg->getUniform("P"), 1, GL_FALSE, value_ptr(P->topMatrix()));
   glUniformMatrix4fv(seaweedProg->getUniform("V"), 1, GL_FALSE, value_ptr(V->topMatrix()));
   glUniform1f(seaweedProg->getUniform("sysTime"), sysTime);

   for (int i = 0; i < seaweedNum; i++) {   
	   MV->pushMatrix();
		  MV->loadIdentity();
		  MV->translate(seaweedPositions[i]);
		  MV->rotate(M_PI/2.0f, vec3(0, 0, 1));
		  MV->rotate(seaweedRotations[i], vec3(1, 0, 0));
		  MV->pushMatrix();
			MV->scale(vec3(8, 0, 6));
		    glUniformMatrix4fv(seaweedProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	plane->draw(seaweedProg);
	   	MV->popMatrix();
	   MV->popMatrix();

	   MV->pushMatrix();
		  MV->loadIdentity();
		  MV->translate(seaweedPositions[i]);
		  MV->rotate(M_PI/2.0f, vec3(0, 0, 1));
		  MV->rotate(seaweedRotations[i]/4.0f, vec3(1, 0, 0));
		  MV->pushMatrix();
			MV->scale(vec3(8, 0, 6));
		    glUniformMatrix4fv(seaweedProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
		  	plane->draw(seaweedProg);
	   	MV->popMatrix();
	   MV->popMatrix();
   }
   seaweedProg->unbind();

   /////////////////////////////////////////////////////////////////////////////////
   // tank walls program
   /////////////////////////////////////////////////////////////////////////////////

   aquariumProg->bind();
   waterTex->bind(aquariumProg->getUniform("waterTex"));


   	glUniformMatrix4fv(aquariumProg->getUniform("V"), 1, GL_FALSE, value_ptr(V->topMatrix()));
   	V->popMatrix();

	glUniformMatrix4fv(aquariumProg->getUniform("P"), 1, GL_FALSE, value_ptr(P->topMatrix()));
	glUniform2f(aquariumProg->getUniform("resolution"), 1, 1);
	glUniform1f(aquariumProg->getUniform("sysTime"), sysTime);
	glUniform1i(aquariumProg->getUniform("dark"), dark);
	glUniform1i(aquariumProg->getUniform("topRipple"), 1);
	glDepthMask(GL_FALSE);
 	//top wall
   MV->pushMatrix();
   	MV->loadIdentity();
   	MV->translate(vec3(0, 20.1, -10));
   	MV->pushMatrix();
   		MV->scale(vec3(30, 0, 20));
   		glUniformMatrix4fv(aquariumProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()) );
	  	plane->draw(aquariumProg);
   	  MV->popMatrix();
   MV->popMatrix();

   glUniform1i(aquariumProg->getUniform("topRipple"), 0);

   //front wall
   MV->pushMatrix();
   	MV->loadIdentity();
   	MV->translate(vec3(0, 10.1, 10));
   	MV->rotate(M_PI/2.0f, vec3(1, 0, 0));
   	MV->pushMatrix();
   		MV->scale(vec3(30, 0, 10));
   		glUniformMatrix4fv(aquariumProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()));
	  	plane->draw(aquariumProg);
   	  MV->popMatrix();
   MV->popMatrix();

   //back wall
   MV->pushMatrix();
   	MV->loadIdentity();
   	MV->translate(vec3(0, 10.1, -30));
   	MV->rotate(-1*M_PI/2.0f, vec3(1, 0, 0));
   	MV->pushMatrix();
   		MV->scale(vec3(30, 0, 10));
   		glUniformMatrix4fv(aquariumProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()));
	  	plane->draw(aquariumProg);
   	  MV->popMatrix();
   MV->popMatrix();

   // //right wall
   MV->pushMatrix();
   	MV->loadIdentity();
   	MV->translate(vec3(30, 10.1, -10));
   	MV->rotate(-1*M_PI/2.0f, vec3(0, 0, 1));
   	MV->pushMatrix();
   		MV->scale(vec3(10, 0, 20));
   		glUniformMatrix4fv(aquariumProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()));
	  	plane->draw(aquariumProg);
   	  MV->popMatrix();
   MV->popMatrix();

   // //left wall
    MV->pushMatrix();
   	MV->loadIdentity();
   	MV->translate(vec3(-30, 10.1, -10));
   	MV->rotate(-1*M_PI/2.0f, vec3(0, 0, 1));
   	MV->pushMatrix();
   		MV->scale(vec3(10, 0, 20));
   		glUniformMatrix4fv(aquariumProg->getUniform("MV"), 1, GL_FALSE,value_ptr(MV->topMatrix()));
	  	plane->draw(aquariumProg);
   	  MV->popMatrix();
   MV->popMatrix();

   aquariumProg->unbind();
   glDepthMask(GL_TRUE);

   P->popMatrix();

}

/* helper function */
static void error_callback(int error, const char *description)
{
	cerr << description << endl;
}

// check parameters for camera inside/outside of tank
static bool checkEyePosition(vec3 oldEye) {
	if (outsideTank) {
		return oldEye.y > 0 && (!(oldEye.z > -30 && oldEye.z < 10 && oldEye.x > -35 && oldEye.x < 35) || oldEye.y > 20);
	} else {
		return oldEye.y > 0 && oldEye.y < 20 && oldEye.z > -29 && oldEye.z < 9 && oldEye.x > -30 && oldEye.x < 29;
	}
}

// toggles between day and night
static void toggleDarkness() {
	if (dark) {
		glClearColor(1.0, 1.0, 1.0, 1.0);
		dark = 0;
	} else {
		glClearColor(0.0, 0.0, 0.0, 1.0);
		dark = 1;
	}
}

// changes the texture on the tank floor
static void changeTankFloor() {
	pebbleIdx++;
	pebbleTex->setFilename(RESOURCE_DIR + "pebbles" + std::to_string(pebbleIdx%4+1) + ".jpg");
	pebbleTex->init();
	pebbleTex->setUnit(1);
}

// toggles between being inside/outside the tank
static void toggleOutsideTank() {
	if (outsideTank) {
		outsideTank = false;
		eye = vec3(0, 20, 8);
		lookAtVec = vec3(0, -0.8, -1);
	} else {
		outsideTank = true;
		eye = vec3(0, 45, 40);
		lookAtVec = vec3(0, -0.8, -1);
	}
	phi = -(M_PI/4.0f);
	theta = (M_PI*3)/2.0f;
}

/* key callback */
static void key_callback(GLFWwindow *window, int key, int scancode, int action, int mods)
{
	if(key == GLFW_KEY_ESCAPE && action == GLFW_PRESS) {
		glfwSetWindowShouldClose(window, GL_TRUE);
	} 
	else if (key == GLFW_KEY_E && (action == GLFW_PRESS || action == GLFW_REPEAT)) {  //move light left
		xLightPos -= 1;
	} 
	else if (key == GLFW_KEY_R && (action == GLFW_PRESS || action == GLFW_REPEAT)) {  //move light right
		xLightPos += 1;
	} 
	else if (key == GLFW_KEY_T && (action == GLFW_PRESS || action == GLFW_REPEAT)) {  //move light down
		yLightPos -= 1;
	} 
	else if (key == GLFW_KEY_Y && (action == GLFW_PRESS || action == GLFW_REPEAT)) {  //move light up
		yLightPos += 1;
	} 
	else if (key == GLFW_KEY_W && (action == GLFW_PRESS || action == GLFW_REPEAT)) {  //move forward
		vec3 oldEye = eye;
		oldEye += vecW*speed;
		if (checkEyePosition(oldEye)) {
			eye = oldEye;
		}
	} 
	else if (key == GLFW_KEY_S && (action == GLFW_PRESS || action == GLFW_REPEAT)) {  //move backward
		vec3 oldEye = eye;
		oldEye -= vecW*speed;
		if (checkEyePosition(oldEye)) {
			eye = oldEye;
		}
	} 
	else if (key == GLFW_KEY_D && (action == GLFW_PRESS || action == GLFW_REPEAT)) {  //move right
		vec3 oldEye = eye;
		oldEye += vecU * speed;
		if (checkEyePosition(oldEye)) {
			eye = oldEye;
		}
	} 
	else if (key == GLFW_KEY_A && (action == GLFW_PRESS || action == GLFW_REPEAT)) {  //move left
		vec3 oldEye = eye;
		oldEye -= vecU * speed;
		if (checkEyePosition(oldEye)) {
			eye = oldEye;
		}
	} 
	else if (key == GLFW_KEY_L && action == GLFW_PRESS) {  //toggle day & night
		toggleDarkness();
	} 
	else if (key == GLFW_KEY_B && action == GLFW_PRESS) {  //change floor texture
		changeTankFloor();
	} 
	else if (key == GLFW_KEY_V && action == GLFW_PRESS) {  //toggle inside & outside tank
		toggleOutsideTank();
	}
}

/* mouse cursor call back */
static void cursor_position_callback(GLFWwindow* window, double xpos, double ypos) {
	double xChange = xpos - xOldMouse;
	double yChange = ypos - yOldMouse;

	theta += 0.01 * xChange;
	phi -= 0.01 * yChange;

	double halfPi = M_PI/ 2.0f;
	double epsilon = 10.0 * M_PI / 180.0;

	phi = phi < -halfPi + epsilon ? -halfPi + epsilon : phi > halfPi - epsilon ? halfPi - epsilon : phi;

	lookAtVec = vec3(
   		cos(phi)*cos(theta),
   		sin(phi),
   		cos(phi)*cos(halfPi - theta));

	xOldMouse = xpos;
	yOldMouse = ypos;
}

/* resize window call back */
static void resize_callback(GLFWwindow *window, int width, int height) {
   g_width = width;
   g_height = height;
   glViewport(0, 0, width, height);
}

//helper function to set materials for shading
void SetMaterial(int i) {
  switch (i) {
    case 1: //magenta - fish
 		glUniform3f(prog->getUniform("MatAmb"), 0.8, 0.0, 0.4);
 		glUniform3f(prog->getUniform("MatDif"), 0.7, 0.3, 0.4);
 		glUniform3f(prog->getUniform("MatSpec"), 0.3, 0.3, 0.4);
 		glUniform1f(prog->getUniform("shine"), 27.9);
      break;
    case 2: //yellow - fish
 		glUniform3f(prog->getUniform("MatAmb"), 1.0, 1.0, 0.0);
 		glUniform3f(prog->getUniform("MatDif"), 0.7804, 0.5686, 0.11373);
 		glUniform3f(prog->getUniform("MatSpec"), 0.1, 0.913, 0.8);
 		glUniform1f(prog->getUniform("shine"), 50);
      break;
	 case 3: //green - fish
 		glUniform3f(prog->getUniform("MatAmb"), 0.1913, 0.8735, 0.3225);
 		glUniform3f(prog->getUniform("MatDif"), 0.2038, 0.27048, 0.1828);
 		glUniform3f(prog->getUniform("MatSpec"), 0.5, 0.5, 0.8);
 		glUniform1f(prog->getUniform("shine"), 18.8);
      break;
      case 4: //orange - fish
		glUniform3f(prog->getUniform("MatAmb"), 1.0, 1.0, 0.0);
 		glUniform3f(prog->getUniform("MatDif"), 0.7804, 0.5686, 0.11373);
 		glUniform3f(prog->getUniform("MatSpec"), 0.1, 0.913, 0.8);
 		glUniform1f(prog->getUniform("shine"), 50);
      break;
      case 5: //purple - fish
		glUniform3f(prog->getUniform("MatAmb"), 0.67508, 0.0, 1.0);
 		glUniform3f(prog->getUniform("MatDif"), 0.2038, 0.1828, 0.7038);
 		glUniform3f(prog->getUniform("MatSpec"), 1.0, 0.913, 0.8);
 		glUniform1f(prog->getUniform("shine"), 18.8);
      break;
      case 6: //blue - fish
		glUniform3f(prog->getUniform("MatAmb"), 0.8784, 0.8784, 0.8784);
 		glUniform3f(prog->getUniform("MatDif"), 0.7038, 0.27048, 0.0828);
 		glUniform3f(prog->getUniform("MatSpec"), 0.0, 0.0, 0.0);
 		glUniform1f(prog->getUniform("shine"), 12.8);
      break;
      case 7: //castle - pink
		glUniform3f(prog->getUniform("MatAmb"), 1.0, 0.5, 0.7);
 		glUniform3f(prog->getUniform("MatDif"), 0.7038, 0.27048, 0.0828);
 		glUniform3f(prog->getUniform("MatSpec"), 1.0, 0.913, 0.8);
 		glUniform1f(prog->getUniform("shine"), 27.8);
      break;
      case 8: //blue - fish
		glUniform3f(prog->getUniform("MatAmb"), 0.5, 0.5, 1.0);
 		glUniform3f(prog->getUniform("MatDif"), 0.0828, 0.27048, 0.7038);
 		glUniform3f(prog->getUniform("MatSpec"), 1.0, 0.913, 0.8);
 		glUniform1f(prog->getUniform("shine"), 27.8);
      break;
      case 9: //black - fish eyes
      	glUniform3f(prog->getUniform("MatAmb"), 0.0, 0.0, 0.0);
 		glUniform3f(prog->getUniform("MatDif"), 0.0828, 0.27048, 0.7038);
 		glUniform3f(prog->getUniform("MatSpec"), 1.0, 0.913, 0.8);
 		glUniform1f(prog->getUniform("shine"), 27.8);
 		break;
 	case 10: //white - fish eyes
      	glUniform3f(prog->getUniform("MatAmb"), 1.0, 1.0, 1.0);
 		glUniform3f(prog->getUniform("MatDif"), 0.0828, 0.27048, 0.7038);
 		glUniform3f(prog->getUniform("MatSpec"), 1.0, 0.913, 0.8);
 		glUniform1f(prog->getUniform("shine"), 27.8);
 		break;
	}
}

int main(int argc, char **argv)
{
	if(argc < 2) {
		cout << "Please specify the resource directory." << endl;
		return 0;
	}
	RESOURCE_DIR = argv[1] + string("/");

	// Set error callback.
	glfwSetErrorCallback(error_callback);
	// Initialize the library.
	if(!glfwInit()) {
		return -1;
	}
   //request the highest possible version of OGL - important for mac
   glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);
   glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
   glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
   glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 2);

	// Create a windowed mode window and its OpenGL context.
	window = glfwCreateWindow(g_width, g_height, "Project 4 - Hilary Schulz", NULL, NULL);
	if(!window) {
		glfwTerminate();
		return -1;
	}
	// Make the window's context current.
	glfwMakeContextCurrent(window);
	// Initialize GLEW.
	glewExperimental = true;
	if(glewInit() != GLEW_OK) {
		cerr << "Failed to initialize GLEW" << endl;
		return -1;
	}
	//weird bootstrap of glGetError
   glGetError();
	cout << "OpenGL version: " << glGetString(GL_VERSION) << endl;
   cout << "GLSL version: " << glGetString(GL_SHADING_LANGUAGE_VERSION) << endl;

	// Set vsync.
	glfwSwapInterval(1);
	// Set keyboard callback.
	glfwSetKeyCallback(window, key_callback);
	glfwSetCursorPosCallback(window, cursor_position_callback);
   //set the window resize call back
   glfwSetFramebufferSizeCallback(window, resize_callback);

	// Initialize scene. Note geometry initialized in init now
	initGL();

	// Loop until the user closes the window.
	while(!glfwWindowShouldClose(window)) {
		// Render scene.
		render();
		// Swap front and back buffers.
		glfwSwapBuffers(window);
		// Poll for and process events.
		glfwPollEvents();
	}
	// Quit program.
	glfwDestroyWindow(window);
	glfwTerminate();
	return 0;
}
